package PI20050411;

import ProSW.*;
import java.util.*;

public abstract class Auto {
	private final String targa, modello;
	private final int nposti;
	private DataOra ultimaManutenzione;
	private final Categoria categoria;

	public Auto(String t, String mod, int np, DataOra um, Categoria cat) throws EccezionePrecondizioni {
		if(t == null) throw new EccezionePrecondizioni("La targa  null");
		targa = t;

		if(mod == null) throw new EccezionePrecondizioni("Il modello  null");
		modello = mod;

		if(np <= 0) throw new EccezionePrecondizioni("Il numero di posti deve essere > 0");
		nposti = np;

		setUltimaManutenzione(um);

		if (cat == null) throw new EccezionePrecondizioni("La categoria non pu essere null");
		categoria = cat;
	}

	public String getTarga() { return targa; }
	public String getModello() { return modello; }
	public int getNPosti() { return nposti; }
	public DataOra getUltimaManutenzione() { return ultimaManutenzione; }
	public void setUltimaManutenzione(DataOra um) throws EccezionePrecondizioni {
		if(um == null) throw new EccezionePrecondizioni("La data di ultima manutenzione  null");
		ultimaManutenzione = um;
	}
	public Categoria getCategoria() { return categoria; }


	public boolean disponibile(DataOra inizio, DataOra fine) throws EccezionePrecondizioni {
		if(inizio == null || fine==null) throw new EccezionePrecondizioni("Gli argomenti non possono essere null");	
		if (fine.primaDi(inizio)) throw new EccezionePrecondizioni("L'intervallo di tempo non  valido");

		try {
			Noleggio n = ultimoNoleggio(fine);
			return n.getFine().primaDiOUgualeA(inizio);
		} catch(EccezionePrecondizioni e) {
			// L'auto non  mai stata noleggiata prima di "fine"
			return true;
		}
	}

	private Noleggio ultimoNoleggio(DataOra momento) throws EccezionePrecondizioni {
		if(momento==null) throw new EccezionePrecondizioni("L'argomento non puo' essere null");		
		Noleggio nmax = null;

		Iterator<TipoLinkAutoNoleggiata> it = linkAutoNoleggiata.iterator();
		while(it.hasNext()) {
			Noleggio n = it.next().getNoleggio();
			if (n.getInizio().primaDiOUgualeA(momento)) {
				if (nmax == null || nmax.getInizio().primaDi(n.getInizio())) {
					nmax = n;
				}
			}
		}
		if (nmax == null) throw new EccezionePrecondizioni("Non ci sono noleggi");
		return nmax;
	}

	public boolean siTrovaIn(DataOra momento, Garage garage) {
		try {
			Noleggio n = ultimoNoleggio(momento);
			return n.getFine().primaDiOUgualeA(momento) && n.getRilascio() == garage;

		} catch(EccezionePrecondizioni e) {
			// Non ci sono noleggi fino a "momento"
			return true;
		}
	}

	public Socio ultimoUtente(DataOra momento) throws EccezionePrecondizioni {
		return ultimoNoleggio(momento).getLinkClienteNoleggio().getSocio();
	}

	// Gestione associazioni: lasciata per esercizio...

} //:~